--[[
	Copyright (C) Udorn (Blackhand)
--]]

qmstr = qmstr or {}

qmstr.QuestMaster = LibStub("AceAddon-3.0"):NewAddon("QuestMaster", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local L = qmstr.Locale.GetInstance()
local log = vendor.Debug:new("QuestMaster")

local YakaGui = LibStub("YakaGui")

local function _CleanupIgnored(self)
	-- TODO cleanup ignored quests and achievements, which are no longer active
end

function _DisplayAutoQuestPopUps(lineFrame, nextAnchor, maxHeight, frameWidth)
	--log:Debug("_DisplayAutoQuestPopUps lineFrame [%s] nextAnchor [%s] maxHeight [%s] frameWidth [%s]", lineFrame, nextAnchor, maxHeight, frameWidth)
	return qmstr.QuestMaster.questWindow:DisplayAutoQuestPopUps(nextAnchor, maxHeight, frameWidth)
end

local function _HideWatchFrame()
	WatchFrameLines:SetScale(0.00001)
	WatchFrameHeader:SetScale(0.00001)
	WatchFrameCollapseExpandButton:SetScale(0.00001)
	
	-- inject own popup handler
	WatchFrame_RemoveObjectiveHandler(WatchFrameAutoQuest_DisplayAutoQuestPopUps)
	WatchFrame_AddObjectiveHandler(_DisplayAutoQuestPopUps)
end

local function _ShowWatchFrame()
	WatchFrameLines:SetScale(1)
	WatchFrameHeader:SetScale(1)
	WatchFrameCollapseExpandButton:SetScale(1)
	
	-- revert popup handler
	WatchFrame_RemoveObjectiveHandler(_DisplayAutoQuestPopUps)
	WatchFrame_AddObjectiveHandler(WatchFrameAutoQuest_DisplayAutoQuestPopUps)
end

local function _CheckScenario(self)
	if (C_Scenario.IsInScenario()) then
		if (not self.inScenario) then
			_ShowWatchFrame()
			self.questWindow:Hide()
			self.inScenario = true
		end
	elseif (self.inScenario) then
		_HideWatchFrame()
		self.questWindow:Show()
		self.inScenario = nil
	end
	self:UpdateEvent(evt)
end

function qmstr.QuestMaster:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("QuestMasterDb", qmstr.Config.DEFAULT_CONFIG, "Default")
end

function qmstr.QuestMaster:OnEnable()

	self.questWindow = qmstr.QuestWindow:new()

	_HideWatchFrame()

	_CleanupIgnored(self)
	
	self:RegisterEvent("QUEST_LOG_UPDATE", "UpdateEvent")
	self:RegisterEvent("QUEST_COMPLETE", "UpdateEvent")
	self:RegisterEvent("PLAYER_LEVEL_UP", "UpdateEvent")
	self:RegisterEvent("CRITERIA_UPDATE", "UpdateEvent")
	self:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE", "UpdateEvent")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("WORLD_MAP_UPDATE")
	self:RegisterEvent("QUEST_POI_UPDATE", "UpdateEvent")
	self:RegisterEvent("QUEST_AUTOCOMPLETE", "UpdateEvent")
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_CLOSE")
	
	self:SecureHook("WatchFrameAutoQuest_ClearPopUp", "UpdateEvent")
	self:SecureHook("RemoveQuestWatch", "UpdateEvent")
	self:SecureHook("AddQuestWatch", "UpdateEvent")
	self:SecureHook("OpenAllBags")
	self:SecureHook("CloseAllBags")
	self:SecureHook("ToggleBackpack")
end

function qmstr.QuestMaster:UpdateEvent(evt)
	log:Debug("UpdateEvent %s", evt)
	self:UpdateWindow()
end

function qmstr.QuestMaster:UpdateWindow(force)
	--log:Debug("UpdateWindow event [%s]", event)
	self.questWindow:Update(force)
end

function qmstr.QuestMaster:SetIgnoreQuest(questId, ignore)
	if (ignore) then
		self.db.profile.ignoredQuests[questId] = true
	else
		self.db.profile.ignoredQuests[questId] = nil
	end
end

function qmstr.QuestMaster:IsQuestIgnored(questId)
	return self.db.profile.ignoredQuests[questId]
end

function qmstr.QuestMaster:SetIgnoreAchievement(achievementId, ignore)
	log:Debug("ignore ach [%s]", achievementId)
	if (ignore) then
		self.db.profile.ignoredAchievements[achievementId] = true
	else
		self.db.profile.ignoredAchievements[achievementId] = nil
	end
end

function qmstr.QuestMaster:IsAchievementIgnored(achievementId)
	return self.db.profile.ignoredAchievements[achievementId]
end

function qmstr.QuestMaster:PlayQuestCompleteSound()
	local sound = self.db.profile.questCompleteSound
	if (strlen(sound) > 0) then
		if (string.find(sound, "SOUND-") == 1) then
			log:Debug("sub [%s]", string.sub(sound, 7))
			PlaySound(string.sub(sound, 7))
		else
			PlaySoundFile(sound)
		end
	end
end

function qmstr.QuestMaster:Reset()
	if (self.questWindow) then
		self.questWindow:Destroy()
	end
	self.questWindow = qmstr.QuestWindow:new()
end

function qmstr.QuestMaster:OpenAllBags()
	log:Debug("OpenAllBags")
	if (self.db.profile.closeMapForBag) then
		if (not self.mapFrameHidden and WorldMapFrame:IsShown()) then
			self.mapFrameHidden = true
			WorldMapFrame:Hide()
		end
	end
end

function qmstr.QuestMaster:CloseAllBags()
	log:Debug("CloseAllBags")
	if (self.db.profile.closeMapForBag) then
		if (self.mapFrameHidden) then
			WorldMapFrame:Show()
			self.mapFrameHidden = nil
		end
	end
end

function qmstr.QuestMaster:ToggleBackpack()
	log:Debug("ToggleBackpack")
	if (self:IsInventoryShown()) then
		self:OpenAllBags()
	else
		self:CloseAllBags()
	end
end

function qmstr.QuestMaster:IsInventoryShown()
	local frame = _G["BagnonFrameinventory"]
	if (frame) then
		return frame:IsShown()
	end
	return IsBagOpen(0)
end

function qmstr.QuestMaster:ChatCommand(msg)
	log:Debug("msg [%s]", msg)
	qmstr.Config:ToogleConfigDialog()
--	InterfaceOptionsFrame_OpenToCategory(self.configFrame)
end

function qmstr.QuestMaster:PET_BATTLE_OPENING_START()
	self.questWindow:Hide()
end

function qmstr.QuestMaster:PET_BATTLE_CLOSE()
	self.questWindow:Show()
end

function qmstr.QuestMaster:ZONE_CHANGED_NEW_AREA(evt)
	_CheckScenario(self);
end

function qmstr.QuestMaster:WORLD_MAP_UPDATE(evt)
	_CheckScenario(self);
end
